<?php

class Criteo_OneTag_Helper_Feed extends Mage_Core_Helper_Abstract
{
    private $feedData;
    private $BASE_URL;
    private $BASE_URL_IMAGE;

    public function __construct()
    {
        $this->feedData = array();
        $this->BASE_URL = Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_WEB);
        $this->BASE_URL_IMAGE = Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_MEDIA);
    }

    public function generateFeed($page = 1, $limit = 250)
    {

        //get product types
        $productTypes = $this->getProductTypesSQL();
        $productIds =  $this->getProductIds($page, $limit, $productTypes);

        $collection = Mage::getResourceModel('catalog/product_collection')
                ->addAttributeToSelect('*')
                ->addFieldToFilter('entity_id', array('in' => $productIds))
                ->addFieldToFilter('status', Mage_Catalog_Model_Product_Status::STATUS_ENABLED);

        //do caching if any
        $categoryNames = $this->getCategoryNames();

        //iterate through resource and create feed
        foreach ($collection as $item) {
            $product = new stdClass();
            try {
                $product->id = $item->getEntityId();
                $product->sku = $item->getSku();

                $product->item_group_id  = '';
                $parentIds = Mage::getModel('catalog/product_type_configurable')->getParentIdsByChild($item->getEntityId());
                if (isset($parentIds[0])) {
                    $product->item_group_id = $parentIds[0];
                }

                $product->title = $item->getName();
                $product->description = $item->getShortDescription();

                $urlPath = $item->getUrlPath();
                if ($urlPath) {
                    $product->link = $this->BASE_URL . $item->getUrlPath();
                } else {
                    $product->link = $item->getProductUrl();
                }

                $product->price = (float) $item->getPrice();
                $product->sale_price = (float) $item->getFinalPrice();
                $product->availability = $item->isAvailable() ? 'in stock' : 'out of stock';

                //$product->image_link = Mage::helper('catalog/image')->init($item, 'image');
                $image_link = $item->getImage();
                if (empty($image_link)) {
                    $image_link = $item->getSmallImage();
                }

                //do not display product with no image
                if (empty($image_link) || $image_link == 'no_selection') {
                    $product->availability = 'out of stock';
                }

                $product->image_link = $this->BASE_URL_IMAGE . 'catalog/product' . $image_link;

                $categoryIds = $item->getCategoryIds();
                $tmpCategoryNames = array();
                foreach ($categoryIds as $id) {
                    array_push($tmpCategoryNames, $categoryNames[$id]);
                }
                $product->product_type = implode(' > ', $tmpCategoryNames);

                array_push($this->feedData, $product);
            } catch (Exception $exc) {
                //is there anything we can do?
            }
        }

        //output XML to browser
        $xml = $this->generateXmlFeed($this->feedData);
        header('Content-type: text/xml; charset=utf-8');
        echo($xml);
    }

    private function generateXmlFeed($feed_data)
    {
        $xmlWriter = new \XMLWriter();
        $xmlWriter->openMemory();
        $xmlWriter->setIndent(true);
        $xmlWriter->setIndentString("    ");
        $xmlWriter->startDocument('1.0', 'UTF-8');
        $xmlWriter->startElement('rss');
        $xmlWriter->writeAttribute('version', '2.0');
        $xmlWriter->writeAttribute('xmlns:g', 'http://base.google.com/ns/1.0');
        $xmlWriter->startElement('channel');
        foreach ($feed_data as $row) {
            $xmlWriter->startElement('item');
            foreach ($row as $key => $value) {
                $xmlWriter->startElement('g:'.$key);
                in_array($key, ['title', 'description']) ? $xmlWriter->writeCData($value) : $xmlWriter->text($value);
                $xmlWriter->endElement(); //g:KEY
            }
            $xmlWriter->endElement(); //item
        }
        $xmlWriter->endElement(); //channel
        $xmlWriter->endElement(); //rss
        return $xmlWriter->flush(true);
    }

    /*
     * Return product types in SQL IN statement
     */
    private function getProductTypesSQL()
    {
        $productTypes = Mage::helper('Criteo_OneTag')->getProductTypes();

        if (empty($productTypes)) {
            $productTypes = 'simple,configurable';
        }
        $productTypes = explode(',', $productTypes);
        $out = '("' . implode('","', $productTypes) . '")';
        return $out;
    }


    /*
     * Return array of product ids
     */
    private function getProductIds($page, $limit, $types)
    {
        $start = ($page - 1) * $limit;
        $dbResource = Mage::getSingleton('core/resource');
        $dbConnection = $dbResource->getConnection('core_read');
        $productTableName = $dbResource->getTableName('catalog/product');

        $query = <<<EOT
            SELECT entity_id FROM $productTableName WHERE type_id IN $types LIMIT $start, $limit
EOT;

        $result = $dbConnection->fetchAll($query);
        $ids = array();

        foreach ($result as $row) {
            array_push($ids, $row['entity_id']);
        }
        return $ids;
    }

    /*
     * Get set of categories name
     */
    private function getCategoryNames()
    {
        $sorted = false;
        $asCollection = true;
        $toLoad = true;
        $names = array();
        $helper = Mage::helper('catalog/category');
        $categories = $helper->getStoreCategories($sorted, $asCollection, $toLoad);

        foreach ($categories as $category) {
            $names[$category->getId()] = $category->getName();
        }
        return $names;
    }
}
